******************************************************************************
*** Program: Inbox2Dbf
*** Time Stamp:   07/31/02 08:26:02 PM
*** Instantiates the custom class defined in this program for automating outlook 2000.
*** Includes functionality to read 
*** messages from the Outlook Inbox and store them in OlMail.dbf
***] WH multiple mods and notes
******************************************************************************
LOCAL loMail, loRootFolder
loMail = CREATEOBJECT( 'sesOutlook' )
IF VARTYPE( loMail ) = 'O'
	WITH loMail
		loRootFolder = .oNameSpace.Folders(1)
		IF .ReadMail( loRootFolder )
			SET DATASESSION TO loMail.DataSessionID
			SELECT SaveMail
			BROWSE LAST
		ENDIF
	ENDWITH
ENDIF
RETURN
			
DEFINE CLASS sesOutlook AS Session

	*** Object reference to the outlook application object
	oOutlook = .NULL.
	*** Object reference to the MAPI namespace object in Outlook from which you reference all the other objects (like the inbox)
	oNamespace = .NULL.

*] 20020911 array for holding contacts
DIMENSION aContacts[1,1]
*] 20020906 name of attachment subdirectory
cDirAttach = "ATTACH\"
*] 20020906 add a counter property
iCounter = 1

  *******************************************************************************************************
	FUNCTION CreateSession()
  *******************************************************************************************************
	*** Instantiate outlook and logon
		LOCAL llRetVal

		*** See we we already have an instance of Outlook Running
		IF TYPE( 'This.oOutlook' ) = 'O' AND NOT ISNULL( This.oOutlook )
			*** No need to create a new instance
		ELSE
			WITH This
				.oOutLook = CREATEOBJECT( 'Outlook.Application' )
				IF TYPE( 'This.oOutLook' ) = 'O' AND NOT ISNULL( .oOutlook )
					.oNameSpace = .oOutlook.GetNameSpace( 'MAPI' )
					IF TYPE( 'This.oNameSpace' ) = 'O' AND NOT ISNULL( .oNameSpace )
						llRetVal = .T.
					ENDIF
				ENDIF
			ENDWITH
		ENDIF

		RETURN llRetVal
	ENDFUNC

  *******************************************************************************************************
	FUNCTION OpenMsgTables()
  *******************************************************************************************************
		WITH This
			IF NOT USED( 'SaveMail' )
				USE SaveMail AGAIN IN 0
			ENDIF
			IF NOT USED( 'OlAttach' )
				USE SaveAttach AGAIN IN 0
			ENDIF
		ENDWITH

*] 20020911
create dbf SaveCont (cnal c(50), cnaf c(50), cnafl c(80), cemail1 c(80), cemail2 c(80), cemail3 c(80))

	ENDFUNC
		
  *******************************************************************************************************
	FUNCTION ReadMail( toFolder )
  *******************************************************************************************************
		*** The ReadMail Method goes through all the Outlook folders recursively and save all 
		*** of the mailItems to the table OlMail.dbf. If the message has any attachments, 
		*** it then calls the SaveAttachments method to save the attachment as a file on disk and 
		*** insert a record with the name of the file into OlAttach.dbf along with a foreign key 
		*** to the message in OlMail.dbf
		LOCAL loItems, loItem, loFolders, loFolder

		*** Get a local referenece to the collection of of items in the current folder
		loItems = toFolder.Items

		*** Process all the items in the current folder
		*** If it is a mail item, save it and process the attachments
		IF VARTYPE( loItems ) = 'O'
			FOR EACH loItem IN loItems
*] 20020906 display counter in a WW
wait window nowait transform(this.iCounter)
				IF 	loItem.Class = 	43		&& olMail	
					*** Add a record to the messages tabel
*] 20020911 store list of recipients as well
*] --> need to add  "omRecip" memo field to SaveMail.DBF

lnCount = loItem.Recipients.Count
m.lcRecip = ''
FOR lnRecip = 1 TO lnCount
  loRecipient = loItem.Recipients[ lnRecip ]
  m.lcRecip = m.lcRecip + loRecipient.Name + ': ' + loRecipient.Address + CHR( 13 ) + CHR( 10 )
ENDFOR

*** 2002-09-12 Marcia G. Akins: let's try a different strategy for matching up the e-mail address
*** First see if the sender name actually IS an e-mail address
IF '@' $ loItem.SenderName
	m.lcSenderEm = loItem.SenderName
ELSE
	lnRow = ASCAN( This.aContacts, loItem.SenderName, -1, -1, 3, 15 )
	IF lnRow > 0
		m.lcSenderEm = This.aContacts[ lnRow, 6 ]
	ELSE
		lnRow = ASCAN( This.aContacts, loItem.SenderName, -1, -1, 4, 15 )
		IF lnRow > 0
			m.lcSenderEm = This.aContacts[ lnRow, 7 ]
		ELSE
			lnRow = ASCAN( This.aContacts, loItem.SenderName, -1, -1, 5, 15 )
			IF lnRow > 0
				m.lcSenderEm = This.aContacts[ lnRow, 8 ]
			ELSE
				m.lcSenderEm = ''
			ENDIF
		ENDIF
	ENDIF
ENDIF

*** First see if the sender name actually IS an e-mail address
IF '@' $ loItem.SenderName
   m.lcSenderEm2 = loItem.SenderName
else
 if empty(loItem.To)
  * don't reply - this is just a draft
  m.lcSenderEm2 = "Draft"
 else
   loReply = loItem.Reply()
   loRecip = loReply.Recipients[ 1 ]
   m.lcSenderEm2 = IIF( NOT EMPTY( loRecip.Address ), loRecip.Address, loRecip.Name )
 endif
ENDIF


					INSERT INTO SaveMail ( omInDate, omSender, omSubject, omBody, omFolder, omRecip, omSenderEm, omSenderEm2 ) ;
						VALUES ( loItem.ReceivedTime, loItem.SenderName, loItem.Subject, loItem.Body, toFolder.Name, m.lcRecip, m.lcSenderEm, m.lcSenderEm2 )
					*** Now see if we have attachments
					IF loItem.Attachments.Count > 0
						This.SaveAttachments( loItem )
					ENDIF
				ENDIF

*] 20020906 increment after each loItem
this.iCounter = this.iCounter + 1

			ENDFOR
		ENDIF

		*** Now see if this folder has folders to process
		IF toFolder.Class = 2	&&	olFolder	
			loFolders = toFolder.Folders
			FOR EACH loFolder IN loFolders
				This.ReadMail( loFolder )
			ENDFOR
		ENDIF
		
		RETURN 
	ENDFUNC

	*******************************************************************************************************
	FUNCTION SaveAttachments( toMsg )
  *******************************************************************************************************
		LOCAL	loAttachment,  lcFileName

		*** Saves the attachmens associated with the current message to the specified file name
		FOR EACH loAttachment IN toMsg.Attachments

*] 20020906 send attachment files into a subdir
* curdir() returns "\AREM\"
* append "ATTACH\"

			lcFileName = FULLPATH( CURDIR() + this.cDirAttach ) + loAttachment.FileName
            if !empty( m.lcFileName )
			loAttachment.SaveAsFile( lcFileName )
			*** Make sure it was saved
			*** The SaveAsFile method returns null...it doesn't tell us about success or failure
			IF FILE( lcFileName )
				*** ok...now add it to the table
				INSERT INTO SaveAttach ( omMailFK, attFname ) VALUES ( SaveMail.omMailPK, lcFileName )
			ENDIF
            endif
		ENDFOR
	ENDFUNC


  *******************************************************************************************************
	FUNCTION Destroy()
  *******************************************************************************************************
		WITH This
			.oNameSpace = .NULL.
			.oOutLook = .NULL.
		ENDWITH
	ENDFUNC


  *******************************************************************************************************
	FUNCTION Init()
  *******************************************************************************************************
		LOCAL llRetVal
		llRetVal = DODEFAULT()
		
		IF llRetVal
			llRetVal = This.CreateSession( )
		ENDIF
		
		IF llRetVal
			This.OpenMsgTables()
		ENDIF

*] 20020911 create the contacts list as well
		IF llRetVal
			This.GetContacts()
		ENDIF

		RETURN llRetVal

	ENDFUNC

  *******************************************************************************************************
	FUNCTION GetContacts()
  *******************************************************************************************************

LOCAL loAddressBook AS Outlook.MAPIFolder, loContact AS Object, lnContactCount

*** Get a reference to the contacts folder
* olFolderContacts is 10
loAddressBook = This.oNameSpace.GetDefaultFolder( 10 )
IF VARTYPE( loAddressBook ) = 'O'
	lnContactCount = 0
	*** Get info about each contact into the array
	FOR EACH loContact IN loAddressBook.Items
		WITH loContact
			*** Make sure we only get individual contacts
			*** and skip any distribution lists
wait window nowait "Parsing Contacts: " + transform(m.lnContactCount) + " processed"
* olContact is 40
			IF .Class = 40
				lnContactCount = lnContactCount + 1
				DIMENSION This.aContacts[ lnContactCount, 9 ]
				This.aContacts[ lnContactCount, 1 ] = .LastName
				This.aContacts[ lnContactCount, 2 ] = .FirstName
				This.aContacts[ lnContactCount, 3 ] = STRTRAN( .Email1DisplayName, '(E-mail)', '', -1, 1, 1  )
				This.aContacts[ lnContactCount, 4 ] = STRTRAN( .Email2DisplayName, '(E-mail 2)', '', -1, 1, 1 )
				This.aContacts[ lnContactCount, 5 ] = STRTRAN( .Email3DisplayName, '(E-mail 3)', '', -1, 1, 1 )
				This.aContacts[ lnContactCount, 6 ] = .Email1Address
				This.aContacts[ lnContactCount, 7 ] = .Email2Address
				This.aContacts[ lnContactCount, 8 ] = .Email3Address
				This.aContacts[ lnContactCount, 9 ] = upper(.FullName)
				m.lcNaf = .LastName
				m.lcNal = .FirstName
				m.lcNafl = .FullName
				m.lcemail1 = .Email1Address
				m.lcemail2 = .Email2Address
				m.lcemail3 = .Email3Address

*] 20020911
insert into SaveCont ;
  (cnaf, cnal, cnafl, cemail1, cemail2, cemail3) ; 
  values ;
  (m.lcnaf, m.lcnal, m.lcNafl, m.lcemail1, m.lcemail2, m.lcemail3)  

			ENDIF
		ENDWITH
	ENDFOR
	ASORT( This.aContacts )
ENDIF


ENDDEFINE
*** EndDefine: sesOutlook

